<?php
namespace JExtstore\Component\JMap\Administrator\Field;
/**  
 * @package JMAP::administrator::components::com_jmap
 * @subpackage Field
 * @author Joomla! Extensions Store
 * @copyright (C) 2021 - Joomla! Extensions Store
 * @license GNU/GPLv2 http://www.gnu.org/licenses/gpl-2.0.html  
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' );
use Joomla\CMS\Form\FormField;
use Joomla\CMS\Language\Text;
use Joomla\CMS\HTML\HTMLHelper;

/**
 * Data sources available
 *
 * @package JMAP::administrator::components::com_jmap
 * @subpackage Field
 */
class RadiocacheField extends FormField {
	/**
	 * The form field type.
	 *
	 * @var    string
	 * @since  11.1
	 */
	protected $type = 'radiocache';

	/**
	 * Method to get the radio button field input markup.
	 *
	 * @return  string  The field input markup.
	 *
	 * @since   11.1
	 */
	protected function getInput()
	{
		// Initialize variables.
		$html = array();
		
		// Initialize some field attributes.
		$class = $this->element['class'] ? ' class="radio btn-group ' . (string) $this->element['class'] . '"' : ' class="radio btn-group pull-left"';
		$translationCacheType = $this->element['cachetype'] ? (string)$this->element['cachetype'] : '';

		// Start the radio field output.
		$html[] = '<fieldset id="' . $this->id . '"' . $class . ' data-bs-toggle="buttons">';

		// Get the field options.
		$options = $this->getOptions();

		// Build the radio field output.
		foreach ($options as $i => $option)
		{

			// Initialize some option attributes.
			$checked = ((string) $option->value == (string) $this->value) ? ' checked="checked"' : '';
			$class = !empty($option->class) ? ' class="' . $option->class . '"' : '';
			$disabled = !empty($option->disable) ? ' disabled="disabled"' : '';

			// Initialize some JavaScript option attributes.
			$onclick = !empty($option->onclick) ? ' onclick="' . $option->onclick . '"' : '';

			$html[] = '<label for="' . $this->id . $i . '"' . $class . '>' .
						'<input type="radio" id="' . $this->id . $i . '" name="' . $this->name . '"' . ' value="' .
						htmlspecialchars($option->value, ENT_COMPAT, 'UTF-8') . '"' . $checked . $class . $onclick . $disabled . '/>' .
						Text::alt($option->text, preg_replace('/[^a-zA-Z0-9_\-]/', '_', $this->fieldname)) . '</label>';
		}

		// End the radio field output.
		$html[] = '</fieldset>';
		
		if($this->value) {
			$html[] = '<span data-bs-content="' . Text::sprintf('COM_JMAP_JOOMLA_' . $translationCacheType . 'CACHE_ENABLED_DESC') . '" class="badge bg-success hasPopover">' .
					  '<span class="fas fa-check" aria-hidden="true"></span>' . Text::sprintf('COM_JMAP_JOOMLA_' . $translationCacheType . 'CACHE_ENABLED') . '</span>';
		} else {
			$html[] = '<span data-bs-content="' . Text::sprintf('COM_JMAP_JOOMLA_' . $translationCacheType . 'CACHE_DISABLED_DESC') . '" class="badge bg-danger hasPopover">' .
					  '<span class="fas fa-exclamation-triangle" aria-hidden="true"></span>' . Text::sprintf('COM_JMAP_JOOMLA_' . $translationCacheType . 'CACHE_DISABLED') . '</span>';
		}
		
		return implode($html);
	}

	/**
	 * Method to get the field options for radio buttons.
	 *
	 * @return  array  The field option objects.
	 *
	 * @since   11.1
	 */
	protected function getOptions()
	{
		// Initialize variables.
		$options = array();

		foreach ($this->element->children() as $option)
		{

			// Only add <option /> elements.
			if ($option->getName() != 'option')
			{
				continue;
			}

			// Create a new option object based on the <option /> element.
			$tmp = HTMLHelper::_(
				'select.option', (string) $option['value'], trim((string) $option), 'value', 'text',
				((string) $option['disabled'] == 'true')
			);

			// Set some option attributes.
			$tmp->class = (string) $option['class'];

			// Set some JavaScript option attributes.
			$tmp->onclick = (string) $option['onclick'];

			// Add the option object to the result set.
			$options[] = $tmp;
		}

		reset($options);

		return $options;
	}
}
