<?php
/**
 * @author Joomla! Extensions Store
 * @package JMAP::plugins::system
 * @copyright (C) 2021 - Joomla! Extensions Store
 * @license GNU/GPLv2 http://www.gnu.org/licenses/gpl-2.0.html
 */
defined ( '_JEXEC' ) or die ( 'Restricted access' );
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\Event\SubscriberInterface;
use Joomla\Event\Event;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Uri\Uri;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Language\LanguageHelper;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Application\ApplicationHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Date\Date;
use Joomla\CMS\Router\Route;
use Joomla\Registry\Registry;
use Joomla\CMS\Filter\OutputFilter;
use Joomla\CMS\Router\SiteRouter;
use Joomla\CMS\Version as JVersion;
use Joomla\CMS\Error\AbstractRenderer;
use Joomla\CMS\Router\Exception\RouteNotFoundException;
use Joomla\String\StringHelper;
use JExtstore\Component\JMap\Administrator\Framework\Loader as JMapLoader;
use JExtstore\Component\JMap\Administrator\Framework\Language\Multilang as JMapMultilang;

/**
 * Observer class notified on events
 *
 * @author Joomla! Extensions Store
 * @package JMAP::plugins::system
 * @since 2.1
 */
class PlgSystemJMap extends CMSPlugin implements SubscriberInterface {
	/**
	 * @access private
	 * @var boolean
	 */
	private $isPluginStopped;
	
	/**
	 * Joomla config object
	 *
	 * @access private
	 * @var Object
	 */
	private $joomlaConfig;
	
	/**
	 * JSitemap config object
	 *
	 * @access private
	 * @var Object
	 */
	private $jmapConfig;
	
	/**
	 * JMap calculate URI
	 *
	 * @access private
	 * @var String
	 */
	private $jmapUri;
	
	/**
	 * App reference
	 *
	 * @access protected
	 * @var Object
	 */
	protected $app;
	
	/**
	 * Get a document object based on the container
	 * 
	 * @param boolean $reset
	 *
	 * @return Document object
	 */
	private function getDocument($reset = false) {
		$container = Factory::getContainer();
		$app = Factory::getApplication();
		
		// Build language object
		$conf = $app->getConfig ();
		$locale = $conf->get ( 'language' );
		$debug = $conf->get ( 'debug_lang' );
		$languageInstance = $container->get ( \Joomla\CMS\Language\LanguageFactoryInterface::class )->createLanguage ( $locale, $debug );
		
		$input = $app->input;
		$type = $input->get ( 'format', 'html', 'cmd' );
		
		// Build document object
		$version = new JVersion ();
		$attributes = array (
				'charset' => 'utf-8',
				'lineend' => 'unix',
				'tab' => "\t",
				'language' => $languageInstance->getTag (),
				'direction' => $languageInstance->isRtl () ? 'rtl' : 'ltr',
				'mediaversion' => $version->getMediaVersion ()
		);
		
		$documentInstance = $container->get ( \Joomla\CMS\Document\FactoryInterface::class )->createDocument ( $type, $attributes );
		
		$reflection = new \ReflectionProperty($app, 'document');
		$reflection->setAccessible(true);
		
		if($reset) {
			$reflection->setValue($app, null);
		} else {
			$reflection->setValue($app, $documentInstance);
		}
		
		return $documentInstance;
	}
	
	/**
	 * Process content plugins
	 * 
	 * @access private
	 * @param string $custom404Text
	 * @param Object &$cParams
	 * @return string
	 */
	private function processContentPlugins($custom404Text, &$cParams) {
		// Process only if html mode is enabled and process plugins param is enabled
		if($cParams->get('custom_404_page_mode', 'html') == 'html' && $cParams->get('custom_404_process_content_plugins', 0)) {
			PluginHelper::importPlugin('content');
			
			$dummyParams = new Registry();
			$elm = new \stdClass();
			$elm->text = $custom404Text;
			
			try {
				// Simulate document creation in this phase
				$doc = $this->getDocument();
				Factory::getApplication()->triggerEvent('onContentPrepare', array ('com_content.article', &$elm, &$dummyParams, 0));
				
				// Reset document to avoid potential issues
				$this->getDocument(true);
				
				// Downgrade the error reporting to avoid the HTTP status code pollution
				$errorReporting = $this->app->get('error_reporting');
				if ($errorReporting === "development" || $errorReporting === "maximum") {
					$this->app->set('error_reporting', 'simple');
				}
			} catch (\Exception $e) {
				$message = $e->getMessage();
			}
			$custom404Text = $elm->text;
		}
		
		// Always return input text, processed or not
		return $custom404Text;
	}
	
	/**
	 * Detect mobile requests
	 *
	 * @access private
	 * @return boolean
	 */
	private function isBotRequest() {
		$crawlers = array (
				'Google' => 'Google',
				'MSN' => 'msnbot',
				'Rambler' => 'Rambler',
				'Yahoo' => 'Yahoo',
				'Yandex' => 'Yandex',
				'AbachoBOT' => 'AbachoBOT',
				'accoona' => 'Accoona',
				'AcoiRobot' => 'AcoiRobot',
				'ASPSeek' => 'ASPSeek',
				'CrocCrawler' => 'CrocCrawler',
				'Dumbot' => 'Dumbot',
				'FAST-WebCrawler' => 'FAST-WebCrawler',
				'GeonaBot' => 'GeonaBot',
				'Gigabot' => 'Gigabot',
				'Lycos spider' => 'Lycos',
				'MSRBOT' => 'MSRBOT',
				'Altavista robot' => 'Scooter',
				'AltaVista robot' => 'Altavista',
				'ID-Search Bot' => 'IDBot',
				'eStyle Bot' => 'eStyle',
				'Scrubby robot' => 'Scrubby',
				'Facebook' => 'facebookexternalhit' 
		);
		// to get crawlers string used in function uncomment it
		// global $crawlers
		if (isset ( $_SERVER ['HTTP_USER_AGENT'] )) {
			$currentUserAgent = $_SERVER ['HTTP_USER_AGENT'];
			// it is better to save it in string than use implode every time
			$crawlers_agents = '/' . implode ( "|", $crawlers ) . '/';
			if (preg_match ( $crawlers_agents, $currentUserAgent, $matches )) {
				return true;
			}
		}
		
		return false;
	}
	
	/**
	 * Handle the addition of the Google Analytics tracking code
	 *
	 * @access private
	 * @param Object $app
	 * @param Object $doc
	 * @param string $location
	 * @return boolean
	 */
	private function addGoogleAnalyticsTrackingCode($app, $doc, $location = 'body') {
		// Get component params
		$injectGaJs = $this->jmapConfig->get ( 'inject_gajs', 0 );
		$gajsCode = trim ( $this->jmapConfig->get ( 'gajs_code', '' ) );
		$gajsVersion = trim ( $this->jmapConfig->get ( 'inject_gajs_version', 'analytics' ) );
		$anonymizeIp = '';
		$anonymizeGtagIp = '';
		if( $this->jmapConfig->get ( 'gajs_anonymize', 0) ) {
			$anonymizeIp = "ga('set', 'anonymizeIp', true);";
			$anonymizeGtagIp = ", { 'anonymize_ip': true }";
		}
		
		// Evaluate nonce csp feature
		$appNonce = $app->get('csp_nonce', null);
		$nonce = $appNonce ? ' nonce="' . $appNonce . '"' : '';
		
		if ($gajsVersion == 'analytics') {
			$script = <<<JS
			<!-- Google Analytics -->
			<script$nonce>
			(function(i,s,o,g,r,a,m){i['GoogleAnalyticsObject']=r;i[r]=i[r]||function(){
			(i[r].q=i[r].q||[]).push(arguments)},i[r].l=1*new Date();a=s.createElement(o),
			m=s.getElementsByTagName(o)[0];a.async=1;a.src=g;m.parentNode.insertBefore(a,m)
			})(window,document,'script','//www.google-analytics.com/analytics.js','ga');
		
			ga('create', '$gajsCode', 'auto');
			ga('send', 'pageview');
			$anonymizeIp
			</script>
			<!-- End Google Analytics -->
JS;
		} elseif ($gajsVersion == 'gtag') {
			$script = <<<JS2
			<!-- Global Site Tag (gtag.js) - Google Analytics -->
			<script$nonce async src="https://www.googletagmanager.com/gtag/js?id=$gajsCode"></script>
			<script$nonce>
			  window.dataLayer = window.dataLayer || [];
			  function gtag(){dataLayer.push(arguments);}
			  gtag('js', new Date());
			  gtag('config', '$gajsCode' $anonymizeGtagIp);
			</script>
JS2;
		}

		// Check if the tracking code must be injected, manipulate output JResponse
		if ($injectGaJs && $gajsCode) {
			if ($location == 'body') {
				$body = $app->getBody ();

				// Replace buffered main view contents at the body end
				$body = preg_replace ( '/<\/body>/i', $script . '</body>', $body, 1 );

				// Set the new JResponse contents
				$app->setBody ( $body );
			} elseif ($location == 'head') {
				if ($doc->getType () === 'html') {
					$doc->addCustomTag ( $script );
				}
			}
		}
	}
	
	/**
	 * Main dispatch method
	 *
	 * @param Event $event
	 * @access public
	 * @return boolean
	 */
	public function dispatchUtility(Event $event) {
		$app = Factory::getApplication ();
		
		// Avoid operations if plugin is executed in backend
		if (!$app->isClient('site')) {
			return;
		}
		
		// Security safe 1 - If JMAP internal link force always the lang url param using the cookie workaround
		if( $app->input->get ( 'option' ) == 'com_jmap' && $this->jmapConfig->get('advanced_multilanguage', 0)) {
			$lang = $app->input->get('lang');
		
			$sefs = LanguageHelper::getLanguages('sef');
			$lang_codes = LanguageHelper::getLanguages('lang_code');
		
			if (isset($sefs[$lang])) {
				$lang_code = $sefs[$lang]->lang_code;
		
				// Create a cookie.
				$conf = $app->getConfig();
				$cookie_domain 	= $conf->get('config.cookie_domain', '');
				$cookie_path 	= $conf->get('config.cookie_path', '/');
				setcookie(ApplicationHelper::getHash('language'), $lang_code, 86400, $cookie_path, $cookie_domain);
				$app->input->cookie->set(ApplicationHelper::getHash('language'), $lang_code);
		
				// Set the request var.
				$app->input->set('language', $lang_code);
				
				// Check if remove default prefix is active and the default language is not the current one
				$defaultSiteLanguage = ComponentHelper::getParams('com_languages')->get('site', 'en-GB');
				$pluginLangFilter = Joomla\CMS\Plugin\PluginHelper::getPlugin('system', 'languagefilter');
				$removeDefaultPrefix = @json_decode($pluginLangFilter->params)->remove_default_prefix;
				if($removeDefaultPrefix && $defaultSiteLanguage != $lang_code) {
					$uri = Uri::getInstance();
					$path = $uri->getPath();
					// Force the language SEF code in the path
					$path = str_replace('/index.php', '/' . $lang . '/index.php', $path);
					$uri->setPath($path);
				}
			}
		}
		
		// Detect if current request come from a bot user agent
		if ($this->isBotRequest () && $app->input->get ( 'option' ) == 'com_jmap') {
			$_SERVER ['REQUEST_METHOD'] = 'POST';
			
			// Set dummy nobot var
			$app->input->post->set ( 'nobotsef', true );
			$GLOBALS['_' . strtoupper('post')] ['nobotsef'] = true;
		}
	}
	
	/**
	 * Hook for the auto Pingomatic third party extensions that have not its own
	 * route helper and work with the universal JSitemap route helper framework
	 *
	 * @param Event $event
	 * @access public
	 * @return boolean
	 */
	public function thirdPartyRoutePinging(Event $event) {
		$this->app = Factory::getApplication ();
		
		// Security safe 2 - If JMAP internal link revert back the query string 'lang' param to the sef lang code 'en' instead of the iso lang code 'en-GB'
		$lang = $this->app->input->get('lang');
		if( $this->app->input->get ( 'option' ) == 'com_jmap' && $this->jmapConfig->get('advanced_multilanguage', 0) && strlen($lang) > 2) {
			$languageCode = $this->app->input->get('language');
			$lang_codes = LanguageHelper::getLanguages('lang_code');
			if(isset($lang_codes[$languageCode])) {
				$sefLang = $lang_codes[$languageCode]->sef;
				$this->app->input->set('lang', $sefLang);
			}
		}
		
		// Avoid below operations if the plugin is not executed in backend app
		if (!$this->app->isClient('administrator')) {
			return;
		}
		
		// Redirect to the component configuration if the Joomla global configuration is requested instead
		$dispatchedComponent = $this->app->input->get ( 'option' );
		$dispatchedView = $this->app->input->get ( 'view' );
		$componentConfig = $this->app->input->get ( 'component' );
		if($dispatchedComponent == 'com_config' && $dispatchedView == 'component' && $componentConfig == 'com_jmap') {
			$this->app->redirect(Route::_('index.php?option=com_jmap&task=config.display', false));
			return;
		}
		
		// Get component params
		$this->cParams = ComponentHelper::getParams ( 'com_jmap' );
		if (! $this->cParams->get ( 'default_autoping', 0 ) && ! $this->cParams->get ( 'autoping', 0 )) {
			return;
		}
		
		// Retrieve more informations as much as possible from the current POST array
		$option = $this->app->input->get ( 'option' );
		$view = $this->app->input->get ( 'view' );
		$controller = $this->app->input->get ( 'controller' );
		$task = $this->app->input->get ( 'task' );
		$id = $this->app->input->getInt ( 'id' );
		$catid = $this->app->input->get ( 'cid', null, 'array' );
		$language = $this->app->input->get ( 'language' );
		$name = $this->app->input->getString ( 'name' );
		if (is_array ( $catid )) {
			$catid = $catid [0];
		}
		
		// Valid execution mapping
		$arrayExecution = array (
				'com_zoo' => array (
						'controller' => 'item',
						'task' => array (
								'apply',
								'save',
								'save2new',
								'save2copy' 
						) 
				) 
		);
		
		// Test against valid execution, discard all invalid extensions operations
		if (array_key_exists ( $option, $arrayExecution )) {
			$testIfExecute = $arrayExecution [$option];
			foreach ( $testIfExecute as $property => $value ) {
				$evaluated = $$property;
				
				if (is_array ( $value )) {
					if (! in_array ( $evaluated, $value )) {
						return;
					}
				} else {
					if ($evaluated != $value) {
						return;
					}
				}
			}
		} else {
			return;
		}
		
		// Valid execution success! Go on to route the request to the content plugin, mimic the native Joomla onContentAfterSave
		
		// Auto loader setup
		// Register autoloader prefix
		require_once JPATH_ADMINISTRATOR . '/components/com_jmap/Framework/Loader.php';
		JMapLoader::setup();
		JMapLoader::registerNamespacePsr4 ( 'JExtstore\\Component\\JMap\\Administrator', JPATH_ADMINISTRATOR . '/components/com_jmap' );
		
		Joomla\CMS\Plugin\PluginHelper::importPlugin ( 'content', 'pingomatic' );
		
		// Simulate the jsitemap_category_id object for the JSitemap route helper
		$elm = new \stdClass ();
		$zooParams = $this->app->input->get ( 'params', null, 'array' );
		$elm->jsitemap_category_id = (int)$zooParams['primary_category'];
		
		// Simulate the $article Joomla object passed to the content observers
		$itemObject = new \stdClass ();
		$itemObject->id = $id;
		$itemObject->catid = $elm;
		$itemObject->option = $option;
		$itemObject->view = $view ? $view : $controller;
		$itemObject->language = $language;
		$itemObject->title = $name;
		
		// Trigger the content plugin event
		$this->app->triggerEvent ( 'onContentAfterSave', array (
				'com_zoo.item',
				$itemObject,
				false 
		) );
	}

	/**
	 * Hook for the management injection of the custom meta tags informations
	 *
	 * @param Event $event
	 * @access public
	 * @return void
	 */
	public function addHeadMetainfo(Event $event) {
		$app = Factory::getApplication ();
		$document = $app->getDocument();

		// Avoid operations if plugin is executed in backend
		if (!$app->isClient('site')) {
			return;
		}

		// Checkpoint for Google Analytics tracking code addition
		if($this->jmapConfig->get('inject_gajs_location', 'body') == 'head') {
			$this->addGoogleAnalyticsTrackingCode($app, $document, 'head');
		}
		
		// Check if the robots opt-in directive is required
		if($this->jmapConfig->get('optin_contents', 0)) {
			$robots = trim($this->jmapConfig->get('optin_contents_robots_directive', 'max-snippet:-1,max-image-preview:large,max-video-preview:-1'));
			if($robots) {
				$currentMetaData = $document->getMetaData('robots');
				if(StringHelper::strpos($currentMetaData, 'noindex') === false && StringHelper::strpos($currentMetaData, 'nosnippet') === false) {
					$document->setMetaData('robots', $robots);
				}
			}
		}
		
		// Get the current URI and check for an entry in the DB table
		if($this->jmapConfig->get('metainfo_urldecode', 1)) {
			$uri = urldecode(Uri::current());
		} else {
			// Preserve URLs character encoding if any
			$uri = Uri::current();
		}

		// K2 Tags decoding as an optional processing
		if($this->jmapConfig->get('metainfo_urlencode_space', 0)) {
			$uri = StringHelper::str_ireplace(' ', '%20', $uri);
		}

		// Remove trailing slash for URL matching, this ensure home page match if the option to remove ending slash is enabled
		if($this->jmapConfig->get('metainfo_remove_trailing_slash', 0)) {
			$uri = rtrim( $uri, '/' );
		}
		
		// Apply same metadata even to the corresponding AMP pages
		if($this->jmapConfig->get('amp_sitemap_enabled', 0)) {
			$ampSuffix = $this->jmapConfig->get('amp_suffix', 'amp');
			if(preg_match("/\.$ampSuffix\./i", $uri)) {
				$uri = preg_replace("/\.$ampSuffix\./i", '.', $uri, 1);
			}
			if(preg_match('/\/' . $ampSuffix . '$/i', $uri)) {
				$uri = preg_replace('/\/' . $ampSuffix . '$/i', '', $uri, 1);
			}
		}
		
		// Store for later stage processing
		$this->jmapUri = $uri;
		
		// Setup the query
		$db = Factory::getContainer()->get('DatabaseDriver');
		$query = "SELECT *" .
				 "\n FROM #__jmap_metainfo" .
				 "\n WHERE " . $db->quoteName('linkurl') . " = " . $db->quote($uri) .
				 "\n AND " . $db->quoteName('published') . " = 1";
		try {
			$metaInfoForThisUri = $db->setQuery($query)->loadObject();
		} catch(\Exception $e) {}

		// Yes! Found some metainfo set for this uri, let's inject them into the document
		if(isset($metaInfoForThisUri->id)) {
			$title = trim($metaInfoForThisUri->meta_title);
			$description = trim($metaInfoForThisUri->meta_desc);
			$image = trim($metaInfoForThisUri->meta_image);
			$robots = $metaInfoForThisUri->robots;
			$ogTagsInclude = $this->jmapConfig->get('metainfo_ogtags', 1);
			$twitterCardsTagsInclude = $this->jmapConfig->get('metainfo_twitter_card_enable', 0);

			// Title and og:graph title
			if($title) {
				// Append site name
				if ($app->get('sitename_pagetitles', 0) == 2 && trim($app->get('sitename'))) {
					$title = $title . ' - ' . trim($app->get('sitename'));
				} elseif ($app->get('sitename_pagetitles', 0) == 1 && trim($app->get('sitename'))) { // Prepend site name
					$title = trim($app->get('sitename')) . ' - ' . $title;
				}
				
				$document->setTitle($title);
				$document->setMetaData('title', $title);
				$document->setMetaData('metatitle', $title);
				if($ogTagsInclude) {
					$document->setMetaData('og:title', $title, 'property');
					$document->setMetaData('twitter:title', $title);
				}
			}

			// Description and og:graph meta description
			if($description) {
				$document->setDescription($description);
				if($ogTagsInclude) {
					$document->setMetaData('og:description', $description, 'property');
					$document->setMetaData('twitter:description', $description);
				}
			}

			// Set always social share uri if title/desc is specified
			if(($title || $description) && $ogTagsInclude) {
				$document->setMetaData('og:url', $uri, 'property');
				$document->setMetaData('og:type', 'article', 'property');
			}
			
			// Image for social share og:image and twitter:image
			if($image && $ogTagsInclude) {
				$imageLink = preg_match('/http/i', $image) ? $image : Uri::base() . ltrim($image, '/');
				$document->setMetaData('og:image', $imageLink, 'property');
				$document->setMetaData('twitter:image', $imageLink);
			}

			// Robots directive
			if($robots) {
				$document->setMetaData('robots', $robots);
			}
			
			// Additional Twitter cards tags
			if($ogTagsInclude && $twitterCardsTagsInclude) {
				$document->setMetaData('twitter:card', 'summary');
				$twitterCardSite = trim($this->jmapConfig->get('metainfo_twitter_card_site', ''));
				if($twitterCardSite) {
					$document->setMetaData('twitter:site', $twitterCardSite);
				}
				$twitterCardCreator = trim($this->jmapConfig->get('metainfo_twitter_card_creator', ''));
				if($twitterCardCreator) {
					$document->setMetaData('twitter:creator', $twitterCardCreator);
				}
			}
		}
		
		// Check if the override canonical feature is enabled and if so go on and check a url matching for some custom canonical
		if($this->jmapConfig->get('seospider_override_canonical', 1)) {
			$query = "SELECT *" .
					 "\n FROM #__jmap_canonicals" .
					 "\n WHERE " . $db->quoteName('linkurl') . " = " . $db->quote($uri);
			try {
				$canonicalForThisUri = $db->setQuery($query)->loadObject();
			} catch(\Exception $e) {}
			
			// Yes! Found a canonical override set for this uri, let's replace them into the document
			if(isset($canonicalForThisUri->id)) {
				// Remove the current canonical tag from the document
				$header = $document->getHeadData();
				foreach($header['links'] as $key => $array) {
					if($array['relation'] == 'canonical') {
						unset($document->_links[$key]);
					}
				}
				
				// Add the new overridden canonical link
				$document->addHeadLink(htmlspecialchars($canonicalForThisUri->canonical), 'canonical', 'rel', array('data-jmap-canonical-override'=>1));
				
				// Override also the og:url metatag with the current canonical
				if(isset($title) && ($title || $description) && $ogTagsInclude) {
					$document->setMetaData('og:url', $canonicalForThisUri->canonical, 'property');
				}
			}
		}
		
		// Fix pagination links if detected adding a page number/results suffix to make them unique and not duplicated
		$isPagination = $app->input->get->get('start', null, 'int');
		$isPage = $app->input->get->get('page', null, 'int');
		if($isPagination || $isPage) {
			$jmapParams = ComponentHelper::getParams('com_jmap');

			// Fix pagination is enabled
			if($jmapParams->get('unique_pagination', 1)) {
				// Get dispatched component params with view overrides
				$contentParams = $app->getParams();

				// Load JMap language translations
				$jLang = $app->getLanguage ();
				$jLang->load ( 'com_jmap', JPATH_ROOT . '/components/com_jmap', 'en-GB', true, true );
				if ($jLang->getTag () != 'en-GB') {
					$jLang->load ( 'com_jmap', JPATH_SITE, null, true, false );
					$jLang->load ( 'com_jmap', JPATH_SITE . '/components/com_jmap', null, true, false );
				}

				// Check if pagination params are detected otherwise fallback
				$leadingNum = $contentParams->get('num_leading_articles', null);
				$introNum = $contentParams->get('num_intro_articles', null);
				if($leadingNum && $introNum) {
					$articlesPerPage = (int)($leadingNum + $introNum);
					$pageNum = ' - ' . Text::_('COM_JMAP_PAGE_NUMBER') . ((int)($isPagination / $articlesPerPage) + 1);
				} else {
					// Fallback for generic components staring from xxx
					if($isPage) {
						$pageNum = ' - ' . Text::_('COM_JMAP_PAGE_NUMBER') . (int)$isPage;
					} else {
						$pageNum = ' - ' . Text::_('COM_JMAP_RESULTS_FROM') . $isPagination;
					}
				}

				$currentTitle = $document->getTitle();
				$document->setTitle($currentTitle . $pageNum);
				$currentDescription = $document->getDescription();
				$document->setDescription($currentDescription . $pageNum);
			}
		}
		
		// Add script json+ld for Rich Snippet Searchbox ONLY to the website homepage
		if ($this->jmapConfig->get ( 'searchbox_enable', 0 )) {
			$loadedMenuItem = $app->getMenu()->getActive();
			if(	$loadedMenuItem && $loadedMenuItem->home == 1 ) {
				$json = array ();
				$array = array ();
				$url = $this->jmapConfig->get ( 'searchbox_url', Uri::root () );
				$type = $this->jmapConfig->get ( 'searchbox_type', 'finder' );
				$custom = $this->jmapConfig->get ( 'searchbox_custom', '' );
				$uriInstance = Uri::getInstance();
				$getDomain = rtrim($uriInstance->getScheme() . '://' . $uriInstance->getHost(), '/');
				
				// Register autoloader prefix
				require_once JPATH_ADMINISTRATOR . '/components/com_jmap/Framework/Loader.php';
				JMapLoader::setup();
				JMapLoader::registerNamespacePsr4 ( 'JExtstore\\Component\\JMap\\Administrator', JPATH_ADMINISTRATOR . '/components/com_jmap' );
				$multiLanguageEnabled = JMapMultilang::isEnabled ();
				$currentSefLanguage = JMapMultilang::getCurrentSefLanguage () . '/';
					
				if ($type == 'finder') {
					$smartSearchComponentLink = Route::_ ( 'index.php?option=com_finder&q={search_term}', false );
					$smartSearchComponentLink = StringHelper::str_ireplace ( '/?', '?', $smartSearchComponentLink );
					if ($multiLanguageEnabled) {
						$smartSearchComponentLink = StringHelper::str_ireplace ( $currentSefLanguage, '', $smartSearchComponentLink );
					}
					$search = $getDomain . $smartSearchComponentLink;
				} else {
					$search = $custom;
				}
					
				$array ['@context'] = 'https://schema.org';
				$array ['@type'] = 'WebSite';
				$array ['url'] = $url;
				$array ['potentialAction'] ['@type'] = 'SearchAction';
				$array ['potentialAction'] ['target'] = $search;
				$array ['potentialAction'] ['query-input'] = 'required name=search_term';
					
				$document->getWebAssetManager()->addInlineScript ( json_encode ( $array ), [], ['type' => 'application/ld+json'] );
			}
		}
	}
	
	/**
	 * Hook for the management of the custom 404 page
	 *
	 * @param Event $event
	 * @subparam $errorClass
	 * @access public
	 * @return boolean
	 */
	public function onExceptionHandler404Page(Event $event) {
		static $custom404Handled = false;
		// subparams: $form, $data
		$arguments = $event->getArguments();
		$errorClass = $arguments['subject'];
		
		if($custom404Handled) {
			return false;
		}

		// Mark as handled for next execution cycles
		$custom404Handled = true;

		// Get component params and ensure that the custom 404 page is enabled
		$cParams = ComponentHelper::getParams('com_jmap');
		if(!$cParams->get('custom_404_page_status', 0)) {
			return false;
		}

		// 404 custom page managed as an override by the postProcessParseRule
		if($cParams->get('custom_404_page_override', 1)) {
			return false;
		}

		// Execute only in frontend
		$app = Factory::getApplication ();
		if (!$app->isClient('site')) {
			return false;
		}
		
		// Dispatched format, apply only to html document
		$documentFormat = $app->input->get ( 'format', null );
		if ($documentFormat && $documentFormat != 'html') {
			return false;
		}
		
		// Dispatched template file, ignores component tmpl
		if ($app->input->get ( 'tmpl', null ) === 'component') {
			return false;
		}
		
		if (!$errorClass instanceof RouteNotFoundException) {
			return false;
		}
	
		$documentRenderer = AbstractRenderer::getRenderer('html');
		$document = $documentRenderer->getDocument();
		
		// Evaluate the error code, 404 only is of our interest and ignore everything else
		// Generate and set a new custom error message based on custom text/html
		$custom404Text = $cParams->get('custom_404_page_text', null);

		// Process contents
		$custom404Text = $this->processContentPlugins($custom404Text, $cParams);

		// Check if a strip tags is required
		if($cParams->get('custom_404_page_mode', 'html') == 'text') {
			$custom404Text = strip_tags($custom404Text);
		}

		// Set the new Exception message supporting HTML and hoping that htmlspecialchars in not used by the error.php of the template
		try {
			$reflection = new \ReflectionProperty($errorClass, 'message');
			$reflection->setAccessible(true);
			$reflection->setValue($errorClass, $custom404Text);
		} catch(\Exception $e) {
			$error = $e->getMessage();
		}
	}

	/**
	 * Application event
	 *
	 * @param Event $event
	 * @access public
	 */
	public function refactorAppBody(Event $event) {
		// Framework reference
		$app = Factory::getApplication ();
		$doc = $app->getDocument ();
	
		// Check if the app can start
		if (!$app->isClient('site')) {
			return false;
		}
	
		// Check if the app can start
		if ($doc->getType () !== 'html') {
			return false;
		}
	
		$option = $app->input->get('option', null);
		if ( $option == 'com_jmap' && $app->input->get('format') ) {
			return false;
		}
		
		// Check if the override headings feature is enabled and if so go on and check a url matching for some heading
		if($this->jmapConfig->get('seospider_override_headings', 1)) {
			$db = Factory::getContainer()->get('DatabaseDriver');
			// Search an headings override for this URL
			$query = "SELECT *" .
					 "\n FROM #__jmap_headings" .
					 "\n WHERE " . $db->quoteName('linkurl') . " = " . $db->quote($this->jmapUri);
			try {
				$headingsForThisUri = $db->setQuery($query)->loadObject();
			} catch(\Exception $e) {}
		
			// Yes! Found some headings override set for this uri, let's replace them into the document
			if(isset($headingsForThisUri->id)) {
				// Go on only if there is at least one valid heading override
				if($headingsForThisUri->h1 || $headingsForThisUri->h2 || $headingsForThisUri->h3) {
					// Include DOM parser class
					require_once (JPATH_ROOT . '/plugins/system/jmap/simplehtmldom.php');
		
					$simpleHtmlDomInstance = new \JMapSimpleHtmlDom();
					$simpleHtmlDomInstance->load( $app->getBody () );
		
					// Find and replace the first encountered H1 tag
					if($headingsForThisUri->h1) {
						$domElementsH1 = $simpleHtmlDomInstance->find( 'h1' );
		
						// Replace the original H1 header with the overridden one
						if(isset($domElementsH1[0])) {
							$element = $domElementsH1[0];
							$nodeText = $element->text(true);
							$nodeText = $headingsForThisUri->h1;
							$element->innertext = $nodeText;
							$element->setAttribute('data-jmap-heading-override', 1);
						}
					}
		
					// Find and replace the first encountered H2 tag
					if($headingsForThisUri->h2) {
						$domElementsH2 = $simpleHtmlDomInstance->find( 'h2' );
							
						// Replace the original H2 header with the overridden one
						if(isset($domElementsH2[0])) {
							$element = $domElementsH2[0];
							$nodeText = $element->text(true);
							$nodeText = $headingsForThisUri->h2;
							$element->innertext = $nodeText;
							$element->setAttribute('data-jmap-heading-override', 1);
						}
					}
		
					// Find and replace the first encountered H3 tag
					if($headingsForThisUri->h3) {
						$domElementsH3 = $simpleHtmlDomInstance->find( 'h3' );
							
						// Replace the original H3 header with the overridden one
						if(isset($domElementsH3[0])) {
							$element = $domElementsH3[0];
							$nodeText = $element->text(true);
							$nodeText = $headingsForThisUri->h3;
							$element->innertext = $nodeText;
							$element->setAttribute('data-jmap-heading-override', 1);
						}
					}
		
					$body = $simpleHtmlDomInstance->save();
		
					// Final assignment
					$app->setBody ( $body );
				}
			}
		}
	
		// Checkpoint for Google Analytics tracking code addition
		if($this->jmapConfig->get('inject_gajs_location', 'body') == 'body') {
			$this->addGoogleAnalyticsTrackingCode($app, $doc, 'body');
		}
	}
	
	/**
	 * Preprocess dummy to load language files
	 *
	 * @param Event $event
	 * @subparam Joomla\CMS\Form\Form $form
	 * @subparam Joomla\CMS\Object\CMSObject $data
	 * @access public
	 * @return boolean
	 */
	public function loadModulesLanguageFiles(Event $event) {
		// subparams: $form, $data
		$arguments = $event->getArguments();
		$form = $arguments[0];
		$data = $arguments[1];
		
		// Manage partial language translations if editing modules jmap in backend
		$app = Factory::getApplication ();
		if((($app->input->get('option') == 'com_modules' || $app->input->get('option') == 'com_advancedmodules') &&
			$app->input->get('view') == 'module' &&
			$app->input->get('layout') == 'edit' &&
			$app->isClient ('administrator')) ||
			($app->input->get('option') == 'com_config' &&
			$app->input->get('view') == 'modules' &&
			$app->input->get('id') &&
			$app->isClient ('site'))) {
			$jLang = $app->getLanguage ();
			$jLang->load ( 'com_jmap', JPATH_ADMINISTRATOR . '/components/com_jmap', 'en-GB', true, true );
			if ($jLang->getTag () != 'en-GB') {
				$jLang->load ( 'com_jmap', JPATH_ADMINISTRATOR, null, true, false );
				$jLang->load ( 'com_jmap', JPATH_ADMINISTRATOR . '/components/com_jmap', null, true, false );
			}
		}
		
		// Check if the default merge data source feature is enabled
		$cParams = ComponentHelper::getParams('com_jmap');
		if(!$cParams->get('merge_generic_menu_by_class', 0)) {
			return true;
		}
		
		// Only works on JForms
		if (!($form instanceof \Joomla\CMS\Form\Form)) return true;
		
		// which belong to the following components
		$components_list = array(
				"com_menus.item"
		);
		
		$formName = $form->getName();
		if ($app->isClient('site') || !in_array($formName, $components_list)) return true;
		
		if(!isset($data->type) || (isset($data->type) && $data->type != 'component')) return true;
		
		switch ($formName) {
			case 'com_menus.item':
				$form->load('<form>
								<fields name="params">
									<fieldset name="menu-options">
										<field name="jsitemap_default_datasource" type="radio" label="Default sitemap menu item" description="Choose this menu item, as the default one for the linked component, to merge data taken from a separate data source" layout="joomla.form.field.radio.switcher" default="0" filter="integer">
											<option value="0">JNO</option>
											<option value="1">JYES</option>
										</field>
									</fieldset>
								</fields>
							</form>');
				break;
		}
		
		return true;
	}
	
	/**
	 * Support for new routing throwing 404 exception in the parse function of the base router
	 *
	 * @param &$router Router object
	 * @param &$uri Uri object
	 * @access public
	 * @return boolean
	 */
	public function postProcessParseRule(&$router, &$uri) {
		// Dispatched format, apply only to html document
		$documentFormat = $this->app->input->get ( 'format', null );
		if ($documentFormat && $documentFormat != 'html') {
			return false;
		}
		
		// Dispatched template file, ignores component tmpl
		if ($this->app->input->get ( 'tmpl', null ) === 'component') {
			return false;
		}
		
		$siteRouter = SiteRouter::getInstance ( 'site' );
		$option = $siteRouter->getVar('option') ? : $uri->getVar('option');
	
		// Check if all parts of the URL have been parsed.
		// Otherwise we have an invalid URL
		if ($option == 'com_content' && strlen($uri->getPath()) > 0) {
			// Get component params and ensure that the custom 404 page is enabled
			$cParams = ComponentHelper::getParams('com_jmap');
	
			// Generate and set a new custom error message based on custom text/html
			$custom404Text = $cParams->get('custom_404_page_text', null);
			
			// Process contents
			$custom404Text = $this->processContentPlugins($custom404Text, $cParams);
	
			// Check if a strip tags is required
			if($cParams->get('custom_404_page_mode', 'html') == 'text') {
				$custom404Text = strip_tags($custom404Text);
			}
			
			throw new \Exception($custom404Text, 404);
		}
	}
	
	/**
	 * Event to manipulate the menu item dashboard in backend
	 *
	 * @param Event $event
	 * @subparam   array  &$policy  The privacy policy status data, passed by reference, with keys "published" and "editLink"
	 *
	 * @return  void
	 */
	public function processMenuItemsDashboard(Event $event) {
		// subparams: $policy
		$arguments = $event->getArguments();
		$context = &$arguments[0];
		$items = &$arguments[1];
		
		if(!empty($items) && $context == 'administrator.module.mod_submenu') {
			foreach ($items as &$item) {
				if($item->element == 'com_jmap') {
					$item->title = 'COM_JMAP_DASHBOARD_TITLE';
				}
			}
		}
	}
	
	/** Manage the Joomla updater based on the user license
	 *
	 * @param Event $event
	 * @subparam   string  The $url for the package update download
	 * @subparam   array  The headers array.
	 * @access public
	 * @return void
	 */
	public function jmapUpdateInstall(Event $event) {
		// subparams: &$url, &$headers
		$arguments = $event->getArguments();
		$url = &$arguments[0];
		$headers = &$arguments[1];
		
		$uri 	= Uri::getInstance($url);
		$parts 	= explode('/', $uri->getPath());
		$app = Factory::getApplication();
		if ($uri->getHost() == 'storejextensions.org' && in_array('com_jsitemap.zip', $parts)) {
			// Init as false unless the license is valid
			$validUpdate = false;
				
			// Manage partial language translations
			$jLang = $app->getLanguage();
			$jLang->load('com_jmap', JPATH_BASE . '/components/com_jmap', 'en-GB', true, true);
			if($jLang->getTag() != 'en-GB') {
				$jLang->load('com_jmap', JPATH_BASE, null, true, false);
				$jLang->load('com_jmap', JPATH_BASE . '/components/com_jmap', null, true, false);
			}
				
			// Email license validation API call and &$url building construction override
			$cParams = ComponentHelper::getParams('com_jmap');
			$registrationEmail = $cParams->get('registration_email', null);
				
			// License
			if($registrationEmail) {
				$prodCode = 'jsitemappro';
				$cdFuncUsed = 'str_' . 'ro' . 't' . '13';
	
				// Retrieve license informations from the remote REST API
				$apiResponse = null;
				$apiEndpoint = $cdFuncUsed('uggc' . '://' . 'fgberwrkgrafvbaf' . '.bet') . "/option,com_easycommerce/action,licenseCode/email,$registrationEmail/productcode,$prodCode";
				if (function_exists('curl_init')){
					$ch = curl_init();
					curl_setopt($ch, CURLOPT_URL, $apiEndpoint);
					curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
					$apiResponse = curl_exec($ch);
					curl_close($ch);
				}
				$objectApiResponse = json_decode($apiResponse);
	
				if(!is_object($objectApiResponse)) {
					// Message user about error retrieving license informations
					$app->enqueueMessage(Text::_('COM_JMAP_ERROR_RETRIEVING_LICENSE_INFO'));
				} else {
					if(!$objectApiResponse->success) {
						switch ($objectApiResponse->reason) {
							// Message user about the reason the license is not valid
							case 'nomatchingcode':
								$app->enqueueMessage(Text::_('COM_JMAP_LICENSE_NOMATCHING'));
								break;
	
							case 'expired':
								// Message user about license expired on $objectApiResponse->expireon
								$app->enqueueMessage(Text::sprintf('COM_JMAP_LICENSE_EXPIRED', $objectApiResponse->expireon));
								break;
						}
							
					}
						
					// Valid license found, builds the URL update link and message user about the license expiration validity
					if($objectApiResponse->success) {
						$url = $cdFuncUsed('uggc' . '://' . 'fgberwrkgrafvbaf' . '.bet' . '/XZY1406TSPQnifs3243560923kfuxnj35td1rtt45664f.ugzy');
	
						$validUpdate = true;
						$app->enqueueMessage(Text::sprintf('COM_JMAP_EXTENSION_UPDATED_SUCCESS', $objectApiResponse->expireon));
					}
				}
			} else {
				// Message user about missing email license code
				$app->enqueueMessage(Text::sprintf('COM_JMAP_MISSING_REGISTRATION_EMAIL_ADDRESS', OutputFilter::ampReplace('index.php?option=com_jmap&task=config.display#_licensepreferences')));
			}
				
			if(!$validUpdate) {
				$app->enqueueMessage(Text::_('COM_JMAP_UPDATER_STANDARD_ADVISE'), 'notice');
			}
		}
	}
	
	/**
	 * Returns an array of events this subscriber will listen to.
	 *
	 * @return  array
	 *
	 * @since 4.0.0
	 */
	public static function getSubscribedEvents(): array {
		return [
				'onAfterInitialise' => 'dispatchUtility',
				'onAfterRoute' => 'thirdPartyRoutePinging',
				'onBeforeCompileHead' => 'addHeadMetainfo',
				'onError' => 'onExceptionHandler404Page',
				'onAfterRender' => 'refactorAppBody',
				'onContentPrepareForm' => 'loadModulesLanguageFiles',
				'onPreprocessMenuItems' => 'processMenuItemsDashboard',
				'onInstallerBeforePackageDownload' => 'jmapUpdateInstall'
		];
	}
	
	/**
	 * Override registers Listeners to the Dispatcher
	 * It allows to stop a plugin execution based on the return value of its constructor
	 *
	 * @override
	 * @return  void
	 */
	public function registerListeners() {
		// Check if the plugin has not been stopped by the constructor
		if(!$this->isPluginStopped) {
			parent::registerListeners();
		}
	}
	
	/**
	 * Class constructor, manage params from component
	 *
	 * @access private
	 * @return boolean
	 */
	public function __construct(& $subject, $config) {
		parent::__construct ( $subject, $config );
		
		// Exclude always the api client
		if ($this->app->isClient ('api')) {
			$this->isPluginStopped = true;
			return;
		}
		
		$this->joomlaConfig = Factory::getApplication()->getConfig ();
		$app = Factory::getApplication ();
		
		// Set the error handler for E_ERROR to be the class handleError method.
		$cParams = ComponentHelper::getParams('com_jmap');
		$this->jmapConfig = $cParams;
		if($cParams->get('custom_404_page_status', 0) && $cParams->get('custom_404_page_override', 1) && $app->isClient('site')) {
			// Add compatibility support for new router management
			$joomlaRouter = Factory::getApplication()->getRouter();
			$joomlaRouter->attachParseRule ( array (
					$this,
					'postProcessParseRule'
			), \Joomla\CMS\Router\Router::PROCESS_AFTER );
		}
	}
}